@@ -19,14 +19,12 @@
 
 #include "Minuit2/FCNAdapter.h"
 #include "Minuit2/FumiliFCNAdapter.h"
-#include "Minuit2/FCNGradAdapter.h"
 #include "Minuit2/FunctionMinimum.h"
 #include "Minuit2/MnMigrad.h"
 #include "Minuit2/MnMinos.h"
 #include "Minuit2/MinosError.h"
 #include "Minuit2/MnHesse.h"
 #include "Minuit2/MinuitParameter.h"
-#include "Minuit2/MnUserFcn.h"
 #include "Minuit2/MnPrint.h"
 #include "Minuit2/VariableMetricMinimizer.h"
 #include "Minuit2/SimplexMinimizer.h"
@@ -83,14 +81,13 @@ int ControlPrintLevel()
 void RestoreGlobalPrintLevel(int) {}
 #endif
 
-Minuit2Minimizer::Minuit2Minimizer(ROOT::Minuit2::EMinimizerType type)
-   : Minimizer(), fDim(0), fMinimizer(nullptr), fMinuitFCN(nullptr), fMinimum(nullptr)
+Minuit2Minimizer::Minuit2Minimizer(ROOT::Minuit2::EMinimizerType type) : fDim(0)
 {
    // Default constructor implementation depending on minimizer type
    SetMinimizerType(type);
 }
 
-Minuit2Minimizer::Minuit2Minimizer(const char *type) : Minimizer(), fDim(0), fMinimizer(nullptr), fMinuitFCN(nullptr), fMinimum(nullptr)
+Minuit2Minimizer::Minuit2Minimizer(const char *type) : fDim(0)
 {
    // constructor from a string
 
@@ -118,62 +115,31 @@ void Minuit2Minimizer::SetMinimizerType(ROOT::Minuit2::EMinimizerType type)
    // Set  minimizer algorithm type
    fUseFumili = false;
    switch (type) {
-   case ROOT::Minuit2::kMigrad:
-      // std::cout << "Minuit2Minimizer: minimize using MIGRAD " << std::endl;
-      SetMinimizer(new ROOT::Minuit2::VariableMetricMinimizer());
-      return;
+   case ROOT::Minuit2::kMigrad: fMinimizer = std::make_unique<ROOT::Minuit2::VariableMetricMinimizer>(); return;
    case ROOT::Minuit2::kMigradBFGS:
-      // std::cout << "Minuit2Minimizer: minimize using MIGRAD " << std::endl;
-      SetMinimizer(new ROOT::Minuit2::VariableMetricMinimizer(VariableMetricMinimizer::BFGSType()));
+      fMinimizer = std::make_unique<ROOT::Minuit2::VariableMetricMinimizer>(VariableMetricMinimizer::BFGSType());
       return;
-   case ROOT::Minuit2::kSimplex:
-      // std::cout << "Minuit2Minimizer: minimize using SIMPLEX " << std::endl;
-      SetMinimizer(new ROOT::Minuit2::SimplexMinimizer());
-      return;
-   case ROOT::Minuit2::kCombined: SetMinimizer(new ROOT::Minuit2::CombinedMinimizer()); return;
-   case ROOT::Minuit2::kScan: SetMinimizer(new ROOT::Minuit2::ScanMinimizer()); return;
+   case ROOT::Minuit2::kSimplex: fMinimizer = std::make_unique<ROOT::Minuit2::SimplexMinimizer>(); return;
+   case ROOT::Minuit2::kCombined: fMinimizer = std::make_unique<ROOT::Minuit2::CombinedMinimizer>(); return;
+   case ROOT::Minuit2::kScan: fMinimizer = std::make_unique<ROOT::Minuit2::ScanMinimizer>(); return;
    case ROOT::Minuit2::kFumili:
-      SetMinimizer(new ROOT::Minuit2::FumiliMinimizer());
+      fMinimizer = std::make_unique<ROOT::Minuit2::FumiliMinimizer>();
       fUseFumili = true;
       return;
    default:
       // migrad minimizer
-      SetMinimizer(new ROOT::Minuit2::VariableMetricMinimizer());
+      fMinimizer = std::make_unique<ROOT::Minuit2::VariableMetricMinimizer>();
    }
 }
 
-Minuit2Minimizer::~Minuit2Minimizer()
-{
-   // Destructor implementation.
-   if (fMinimizer)
-      delete fMinimizer;
-   if (fMinuitFCN)
-      delete fMinuitFCN;
-   if (fMinimum)
-      delete fMinimum;
-}
-
-Minuit2Minimizer::Minuit2Minimizer(const Minuit2Minimizer &) : ROOT::Math::Minimizer()
-{
-   // Implementation of copy constructor.
-}
-
-Minuit2Minimizer &Minuit2Minimizer::operator=(const Minuit2Minimizer &rhs)
-{
-   // Implementation of assignment operator.
-   if (this == &rhs)
-      return *this; // time saving self-test
-   return *this;
-}
+Minuit2Minimizer::~Minuit2Minimizer() = default;
 
 void Minuit2Minimizer::Clear()
 {
    // delete the state in case of consecutive minimizations
    fState = MnUserParameterState();
    // clear also the function minimum
-   if (fMinimum)
-      delete fMinimum;
-   fMinimum = nullptr;
+   fMinimum.reset();
 }
 
 // set variables
@@ -206,6 +172,31 @@ bool Minuit2Minimizer::SetVariable(unsigned int ivar, const std::string &name, d
    return true;
 }
 
+/** set initial second derivatives
+ */
+bool Minuit2Minimizer::SetCovarianceDiag(std::span<const double> d2, unsigned int n)
+{
+   MnPrint print("Minuit2Minimizer::SetCovarianceDiag", PrintLevel());
+
+   std::vector<double> cov(n * (n + 1) / 2);
+
+   for (unsigned int i = 0; i < n; i++) {
+      for (unsigned int j = i; j < n; j++)
+         cov[i + j * (j + 1) / 2] = (i == j) ? d2[i] : 0.;
+   }
+
+   return Minuit2Minimizer::SetCovariance(cov, n);
+}
+
+bool Minuit2Minimizer::SetCovariance(std::span<const double> cov, unsigned int nrow)
+{
+   MnPrint print("Minuit2Minimizer::SetCovariance", PrintLevel());
+
+   fState.AddCovariance({cov, nrow});
+
+   return true;
+}
+
 bool Minuit2Minimizer::SetLowerLimitedVariable(unsigned int ivar, const std::string &name, double val, double step,
                                                double lower)
 {
@@ -379,41 +370,48 @@ bool Minuit2Minimizer::GetVariableSettings(unsigned int ivar, ROOT::Fit::Paramet
 void Minuit2Minimizer::SetFunction(const ROOT::Math::IMultiGenFunction &func)
 {
    // set function to be minimized
-   if (fMinuitFCN)
-      delete fMinuitFCN;
+   fMinuitFCN.reset();
    fDim = func.NDim();
    const bool hasGrad = func.HasGradient();
    if (!fUseFumili) {
-      fMinuitFCN = hasGrad ? static_cast<ROOT::Minuit2::FCNBase *>(new ROOT::Minuit2::FCNGradAdapter<ROOT::Math::IMultiGradFunction>(dynamic_cast<ROOT::Math::IMultiGradFunction const&>(func), ErrorDef()))
-                           : static_cast<ROOT::Minuit2::FCNBase *>(new ROOT::Minuit2::FCNAdapter<ROOT::Math::IMultiGenFunction>(func, ErrorDef()));
-   } else {
-      if(hasGrad) {
-         // for Fumili the fit method function interface is required
-         auto fcnfunc = dynamic_cast<const ROOT::Math::FitMethodGradFunction *>(&func);
-         if (!fcnfunc) {
-            MnPrint print("Minuit2Minimizer", PrintLevel());
-            print.Error("Wrong Fit method function for Fumili");
-            return;
-         }
-         fMinuitFCN = new ROOT::Minuit2::FumiliFCNAdapter<ROOT::Math::FitMethodGradFunction>(*fcnfunc, fDim, ErrorDef());
-      } else {
-         // for Fumili the fit method function interface is required
-         auto fcnfunc = dynamic_cast<const ROOT::Math::FitMethodFunction *>(&func);
-         if (!fcnfunc) {
-            MnPrint print("Minuit2Minimizer", PrintLevel());
-            print.Error("Wrong Fit method function for Fumili");
-            return;
-         }
-         fMinuitFCN = new ROOT::Minuit2::FumiliFCNAdapter<ROOT::Math::FitMethodFunction>(*fcnfunc, fDim, ErrorDef());
+      auto lambdaFunc = [&func](double const *params) { return func(params); };
+      auto adapter = std::make_unique<ROOT::Minuit2::FCNAdapter>(lambdaFunc, ErrorDef());
+      if (hasGrad) {
+         auto const &gradFunc = dynamic_cast<ROOT::Math::IMultiGradFunction const &>(func);
+         auto lambdaGrad = [&gradFunc](double const *params, double *grad) { return gradFunc.Gradient(params, grad); };
+         adapter->SetGradientFunction(lambdaGrad);
       }
+      fMinuitFCN = std::move(adapter);
+      return;
+   }
+   if (hasGrad) {
+      // for Fumili the fit method function interface is required
+      auto fcnfunc = dynamic_cast<const ROOT::Math::FitMethodGradFunction *>(&func);
+      if (!fcnfunc) {
+         MnPrint print("Minuit2Minimizer", PrintLevel());
+         print.Error("Wrong Fit method function for Fumili");
+         return;
+      }
+      fMinuitFCN = std::make_unique<ROOT::Minuit2::FumiliFCNAdapter<ROOT::Math::FitMethodGradFunction>>(*fcnfunc, fDim,
+                                                                                                        ErrorDef());
+   } else {
+      // for Fumili the fit method function interface is required
+      auto fcnfunc = dynamic_cast<const ROOT::Math::FitMethodFunction *>(&func);
+      if (!fcnfunc) {
+         MnPrint print("Minuit2Minimizer", PrintLevel());
+         print.Error("Wrong Fit method function for Fumili");
+         return;
+      }
+      fMinuitFCN =
+         std::make_unique<ROOT::Minuit2::FumiliFCNAdapter<ROOT::Math::FitMethodFunction>>(*fcnfunc, fDim, ErrorDef());
    }
 }
 
-void Minuit2Minimizer::SetHessianFunction(std::function<bool(const std::vector<double> &, double *)> hfunc)
+void Minuit2Minimizer::SetHessianFunction(std::function<bool(std::span<const double>, double *)> hfunc)
 {
    // for Fumili not supported for the time being
    if (fUseFumili) return;
-   auto fcn = static_cast<ROOT::Minuit2::FCNGradAdapter<ROOT::Math::IMultiGradFunction> *>(fMinuitFCN);
+   auto fcn = static_cast<ROOT::Minuit2::FCNAdapter *>(fMinuitFCN.get());
    if (!fcn) return;
    fcn->SetHessianFunction(hfunc);
 }
@@ -450,10 +448,16 @@ ROOT::Minuit2::MnStrategy customizedStrategy(unsigned int strategyLevel, ROOT::M
 
 } // namespace
 
+/// Perform the minimization and store a copy of FunctionMinimum.
+/// The maximum number of function calls used can be checked via `this->MaxFunctionCalls()`,
+/// if this value is 0 (the default), then it is replaced with
+/// `2 * (nvar + 1) * (200 + 100 * nvar + 5 * nvar * nvar` where `nvar` is number of variable parameters.
+/// \see MnMinos::FindCrossValue
+/// Other minimization settings can be retrieved via `Tolerance()`, `Strategy()`, `ErrorDef()`, `Precision()`
+/// \see ROOT::Math::MinimizerOptions::PrintDefault()
 bool Minuit2Minimizer::Minimize()
 {
-   // perform the minimization
-   // store a copy of FunctionMinimum
+
 
    MnPrint print("Minuit2Minimizer::Minimize", PrintLevel());
 
@@ -465,9 +469,7 @@ bool Minuit2Minimizer::Minimize()
    assert(GetMinimizer() != nullptr);
 
    // delete result of previous minimization
-   if (fMinimum)
-      delete fMinimum;
-   fMinimum = nullptr;
+   fMinimum.reset();
 
    const int maxfcn = MaxFunctionCalls();
    const double tol = Tolerance();
@@ -475,6 +477,7 @@ bool Minuit2Minimizer::Minimize()
    fMinuitFCN->SetErrorDef(ErrorDef());
 
    const int printLevel = PrintLevel();
+   print.Debug("Minuit print level is", printLevel);
    if (PrintLevel() >= 1) {
       // print the real number of maxfcn used (defined in ModularFunctionMinimizer)
       int maxfcn_used = maxfcn;
@@ -509,6 +512,17 @@ bool Minuit2Minimizer::Minimize()
       if (ret)
          SetStorageLevel(storageLevel);
 
+      // fumili options
+      if (fUseFumili) {
+         std::string fumiliMethod;
+         ret = minuit2Opt->GetValue("FumiliMethod", fumiliMethod);
+         if (ret) {
+            auto fumiliMinimizer = dynamic_cast<ROOT::Minuit2::FumiliMinimizer *>(fMinimizer.get());
+            if (fumiliMinimizer)
+               fumiliMinimizer->SetMethod(fumiliMethod);
+         }
+      }
+
       if (printLevel > 0) {
          std::cout << "Minuit2Minimizer::Minuit  - Changing default options" << std::endl;
          minuit2Opt->Print();
@@ -549,16 +563,8 @@ bool Minuit2Minimizer::Minimize()
 
    const ROOT::Minuit2::MnStrategy strategy = customizedStrategy(strategyLevel, fOptions);
 
-   const ROOT::Minuit2::FCNGradientBase *gradFCN = dynamic_cast<const ROOT::Minuit2::FCNGradientBase *>(fMinuitFCN);
-   if (gradFCN != nullptr) {
-      // use gradient
-      // SetPrintLevel(3);
-      ROOT::Minuit2::FunctionMinimum min = GetMinimizer()->Minimize(*gradFCN, fState, strategy, maxfcn, tol);
-      fMinimum = new ROOT::Minuit2::FunctionMinimum(min);
-   } else {
-      ROOT::Minuit2::FunctionMinimum min = GetMinimizer()->Minimize(*GetFCN(), fState, strategy, maxfcn, tol);
-      fMinimum = new ROOT::Minuit2::FunctionMinimum(min);
-   }
+   ROOT::Minuit2::FunctionMinimum min = GetMinimizer()->Minimize(*fMinuitFCN, fState, strategy, maxfcn, tol);
+   fMinimum = std::make_unique<ROOT::Minuit2::FunctionMinimum>(min);
 
    // check if Hesse needs to be run. We do it when is requested (IsValidError() == true , set by SetParabError(true) in fitConfig)
    // (IsValidError() means the flag to get correct error from the Minimizer is set (Minimizer::SetValidError())
@@ -595,7 +601,7 @@ bool Minuit2Minimizer::ExamineMinimum(const ROOT::Minuit2::FunctionMinimum &min)
    int debugLevel = PrintLevel();
    if (debugLevel >= 3) {
 
-      const std::vector<ROOT::Minuit2::MinimumState> &iterationStates = min.States();
+      std::span<const ROOT::Minuit2::MinimumState> iterationStates = min.States();
       std::cout << "Number of iterations " << iterationStates.size() << std::endl;
       for (unsigned int i = 0; i < iterationStates.size(); ++i) {
          // std::cout << iterationStates[i] << std::endl;
@@ -663,7 +669,7 @@ bool Minuit2Minimizer::ExamineMinimum(const ROOT::Minuit2::FunctionMinimum &min)
       PrintResults();
 
    // set the minimum values in the fValues vector
-   const std::vector<MinuitParameter> &paramsObj = fState.MinuitParameters();
+   std::span<const MinuitParameter> paramsObj = fState.MinuitParameters();
    if (paramsObj.empty())
       return false;
    assert(fDim == paramsObj.size());
@@ -713,7 +719,7 @@ void Minuit2Minimizer::PrintResults()
 const double *Minuit2Minimizer::Errors() const
 {
    // return error at minimum (set to zero for fixed and constant params)
-   const std::vector<MinuitParameter> &paramsObj = fState.MinuitParameters();
+   std::span<const MinuitParameter> paramsObj = fState.MinuitParameters();
    if (paramsObj.empty())
       return nullptr;
    assert(fDim == paramsObj.size());
@@ -828,21 +834,27 @@ double Minuit2Minimizer::Correlation(unsigned int i, unsigned int j) const
    return 0;
 }
 
-double Minuit2Minimizer::GlobalCC(unsigned int i) const
+std::vector<double> Minuit2Minimizer::GlobalCC() const
 {
    // get global correlation coefficient for the parameter i. This is a number between zero and one which gives
    // the correlation between the i-th parameter  and that linear combination of all other parameters which
    // is most strongly correlated with i.
 
-   if (i >= fDim)
-      return 0;
+   std::vector<double> out;
+   MnGlobalCorrelationCoeff globalCC = fState.GlobalCC();
    // no info available when minimization has failed or has some problems
-   if (!fState.HasGlobalCC())
-      return 0;
-   if (fState.Parameter(i).IsFixed() || fState.Parameter(i).IsConst())
-      return 0;
-   unsigned int k = fState.IntOfExt(i);
-   return fState.GlobalCC().GlobalCC()[k];
+   if (!globalCC.IsValid())
+      return out;
+   out.resize(fDim);
+   for (unsigned int i = 0; i < fDim; ++i) {
+      if (fState.Parameter(i).IsFixed() || fState.Parameter(i).IsConst())
+         out[i] = 0;
+      else {
+         unsigned int k = fState.IntOfExt(i);
+         out[i] = globalCC.GlobalCC()[k];
+      }
+   }
+   return out;
 }
 
 bool Minuit2Minimizer::GetMinosError(unsigned int i, double &errLow, double &errUp, int runopt)
@@ -1165,7 +1177,7 @@ bool Minuit2Minimizer::Contour(unsigned int ipar, unsigned int jpar, unsigned in
       fMinimum->SetErrorDef(ErrorDef());
    }
 
-   print.Info("Computing contours -", ErrorDef());
+   print.Info("Computing contours at level -", ErrorDef());
 
    // switch off Minuit2 printing (for level of  0,1)
    const int prev_level = (PrintLevel() <= 1) ? TurnOffPrintInfoLevel() : -2;
@@ -1193,6 +1205,14 @@ bool Minuit2Minimizer::Contour(unsigned int ipar, unsigned int jpar, unsigned in
       x[i] = result[i].first;
       y[i] = result[i].second;
    }
+   print.Info([&](std::ostream &os) {
+                  os << " Computed " << npoints << " points at level " << ErrorDef();
+                  for (unsigned int i = 0; i < npoints; i++) {
+                     if (i %5 == 0) os << std::endl;
+                     os << "( " << x[i] << ", " << y[i] << ") ";
+                  }
+                  os << std::endl << std::endl;
+               });
 
    return true;
 }
@@ -1332,6 +1352,12 @@ void Minuit2Minimizer::SetStorageLevel(int level)
       fMinimizer->Builder().SetStorageLevel(level);
 }
 
+void Minuit2Minimizer::SetFCN(unsigned int nDim, std::unique_ptr<ROOT::Minuit2::FCNBase> fcn)
+{
+   fDim = nDim;
+   fMinuitFCN = std::move(fcn);
+}
+
 } // end namespace Minuit2
 
 } // end namespace ROOT
