@@ -19,39 +19,30 @@ namespace Minuit2 {
 //
 // construct from user parameters (before minimization)
 //
-MnUserParameterState::MnUserParameterState(const std::vector<double> &par, const std::vector<double> &err)
-   : fValid(true), fCovarianceValid(false), fGCCValid(false), fCovStatus(-1), fFVal(0.), fEDM(0.), fNFcn(0),
-     fParameters(MnUserParameters(par, err)), fCovariance(MnUserCovariance()), fGlobalCC(MnGlobalCorrelationCoeff()),
-     fIntParameters(par), fIntCovariance(MnUserCovariance())
+MnUserParameterState::MnUserParameterState(std::span<const double> par, std::span<const double> err)
+   : fValid(true), fCovStatus(-1), fParameters(MnUserParameters(par, err)), fIntParameters(par.begin(), par.end())
 {
 }
 
-MnUserParameterState::MnUserParameterState(const MnUserParameters &par)
-   : fValid(true), fCovarianceValid(false), fGCCValid(false), fCovStatus(-1), fFVal(0.), fEDM(0.), fNFcn(0),
-     fParameters(par), fCovariance(MnUserCovariance()), fGlobalCC(MnGlobalCorrelationCoeff()),
-     fIntParameters(std::vector<double>()), fIntCovariance(MnUserCovariance())
+MnUserParameterState::MnUserParameterState(const MnUserParameters &par) : fValid(true), fCovStatus(-1), fParameters(par)
 {
    // construct from user parameters (before minimization)
 
-   for (std::vector<MinuitParameter>::const_iterator ipar = MinuitParameters().begin();
-        ipar != MinuitParameters().end(); ++ipar) {
-      if ((*ipar).IsConst() || (*ipar).IsFixed())
+   for (auto const &ipar : MinuitParameters()) {
+      if (ipar.IsConst() || ipar.IsFixed())
          continue;
-      if ((*ipar).HasLimits())
-         fIntParameters.push_back(Ext2int((*ipar).Number(), (*ipar).Value()));
+      if (ipar.HasLimits())
+         fIntParameters.push_back(Ext2int(ipar.Number(), ipar.Value()));
       else
-         fIntParameters.push_back((*ipar).Value());
+         fIntParameters.push_back(ipar.Value());
    }
 }
 
 //
 // construct from user parameters + errors (before minimization)
 //
-MnUserParameterState::MnUserParameterState(const std::vector<double> &par, const std::vector<double> &cov,
-                                           unsigned int nrow)
-   : fValid(true), fCovarianceValid(true), fGCCValid(false), fCovStatus(-1), fFVal(0.), fEDM(0.), fNFcn(0),
-     fParameters(MnUserParameters()), fCovariance(MnUserCovariance(cov, nrow)), fGlobalCC(MnGlobalCorrelationCoeff()),
-     fIntParameters(par), fIntCovariance(MnUserCovariance(cov, nrow))
+MnUserParameterState::MnUserParameterState(std::span<const double> par, std::span<const double> cov, unsigned int nrow)
+   : fValid(true), fCovStatus(-1), fIntParameters(par.begin(), par.end())
 {
    // construct from user parameters + errors (before minimization) using std::vector for parameter error and    // an
    // std::vector of size n*(n+1)/2 for the covariance matrix  and n (rank of cov matrix)
@@ -63,13 +54,11 @@ MnUserParameterState::MnUserParameterState(const std::vector<double> &par, const
       err.push_back(std::sqrt(fCovariance(i, i)));
    }
    fParameters = MnUserParameters(par, err);
-   assert(fCovariance.Nrow() == VariableParameters());
+   MnUserParameterState::AddCovariance({cov, nrow});
 }
 
-MnUserParameterState::MnUserParameterState(const std::vector<double> &par, const MnUserCovariance &cov)
-   : fValid(true), fCovarianceValid(true), fGCCValid(false), fCovStatus(-1), fFVal(0.), fEDM(0.), fNFcn(0),
-     fParameters(MnUserParameters()), fCovariance(cov), fGlobalCC(MnGlobalCorrelationCoeff()), fIntParameters(par),
-     fIntCovariance(cov)
+MnUserParameterState::MnUserParameterState(std::span<const double> par, const MnUserCovariance &cov)
+   : fValid(true), fCovStatus(-1), fIntParameters(par.begin(), par.end())
 {
    // construct from user parameters + errors (before minimization) using std::vector (params) and MnUserCovariance
    // class
@@ -81,77 +70,67 @@ MnUserParameterState::MnUserParameterState(const std::vector<double> &par, const
       err.push_back(std::sqrt(fCovariance(i, i)));
    }
    fParameters = MnUserParameters(par, err);
-   assert(fCovariance.Nrow() == VariableParameters());
+   MnUserParameterState::AddCovariance(cov);
 }
 
 MnUserParameterState::MnUserParameterState(const MnUserParameters &par, const MnUserCovariance &cov)
-   : fValid(true), fCovarianceValid(true), fGCCValid(false), fCovStatus(-1), fFVal(0.), fEDM(0.), fNFcn(0),
-     fParameters(par), fCovariance(cov), fGlobalCC(MnGlobalCorrelationCoeff()), fIntParameters(std::vector<double>()),
-     fIntCovariance(cov)
+   : fValid(true), fCovStatus(-1), fParameters(par)
 {
    // construct from user parameters + errors (before minimization) using
    // MnUserParameters and MnUserCovariance objects
 
-   fIntCovariance.Scale(0.5);
-   for (std::vector<MinuitParameter>::const_iterator ipar = MinuitParameters().begin();
-        ipar != MinuitParameters().end(); ++ipar) {
-      if ((*ipar).IsConst() || (*ipar).IsFixed())
+   for (auto const &ipar : MinuitParameters()) {
+      if (ipar.IsConst() || ipar.IsFixed())
          continue;
-      if ((*ipar).HasLimits())
-         fIntParameters.push_back(Ext2int((*ipar).Number(), (*ipar).Value()));
+      if (ipar.HasLimits())
+         fIntParameters.push_back(Ext2int(ipar.Number(), ipar.Value()));
       else
-         fIntParameters.push_back((*ipar).Value());
+         fIntParameters.push_back(ipar.Value());
    }
-   assert(fCovariance.Nrow() == VariableParameters());
-   //
-   // need to Fix that in case of limited parameters
-   //   fIntCovariance = MnUserCovariance();
-   //
+
+   MnUserParameterState::AddCovariance(cov);
 }
 
 //
 //
 MnUserParameterState::MnUserParameterState(const MinimumState &st, double up, const MnUserTransformation &trafo)
-   : fValid(st.IsValid()), fCovarianceValid(false), fGCCValid(false), fCovStatus(-1), fFVal(st.Fval()), fEDM(st.Edm()),
-     fNFcn(st.NFcn()), fParameters(MnUserParameters()), fCovariance(MnUserCovariance()),
-     fGlobalCC(MnGlobalCorrelationCoeff()), fIntParameters(std::vector<double>()), fIntCovariance(MnUserCovariance())
+   : fValid(st.IsValid()), fCovStatus(-1), fFVal(st.Fval()), fEDM(st.Edm()), fNFcn(st.NFcn())
 {
    //
    // construct from internal parameters (after minimization)
    //
    // std::cout << "build a MnUserParameterState after minimization.." << std::endl;
 
-   for (std::vector<MinuitParameter>::const_iterator ipar = trafo.Parameters().begin();
-        ipar != trafo.Parameters().end(); ++ipar) {
-      if ((*ipar).IsConst()) {
-         Add((*ipar).GetName(), (*ipar).Value());
-      } else if ((*ipar).IsFixed()) {
-         Add((*ipar).GetName(), (*ipar).Value(), (*ipar).Error());
-         if ((*ipar).HasLimits()) {
-            if ((*ipar).HasLowerLimit() && (*ipar).HasUpperLimit())
-               SetLimits((*ipar).GetName(), (*ipar).LowerLimit(), (*ipar).UpperLimit());
-            else if ((*ipar).HasLowerLimit() && !(*ipar).HasUpperLimit())
-               SetLowerLimit((*ipar).GetName(), (*ipar).LowerLimit());
+   for (auto const &ipar : trafo.Parameters()) {
+      if (ipar.IsConst()) {
+         Add(ipar.GetName(), ipar.Value());
+      } else if (ipar.IsFixed()) {
+         Add(ipar.GetName(), ipar.Value(), ipar.Error());
+         if (ipar.HasLimits()) {
+            if (ipar.HasLowerLimit() && ipar.HasUpperLimit())
+               SetLimits(ipar.GetName(), ipar.LowerLimit(), ipar.UpperLimit());
+            else if (ipar.HasLowerLimit() && !ipar.HasUpperLimit())
+               SetLowerLimit(ipar.GetName(), ipar.LowerLimit());
             else
-               SetUpperLimit((*ipar).GetName(), (*ipar).UpperLimit());
+               SetUpperLimit(ipar.GetName(), ipar.UpperLimit());
          }
-         Fix((*ipar).GetName());
-      } else if ((*ipar).HasLimits()) {
-         unsigned int i = trafo.IntOfExt((*ipar).Number());
+         Fix(ipar.GetName());
+      } else if (ipar.HasLimits()) {
+         unsigned int i = trafo.IntOfExt(ipar.Number());
          double err =
             st.Error().IsValid() ? std::sqrt(2. * up * st.Error().InvHessian()(i, i)) : st.Parameters().Dirin()(i);
-         Add((*ipar).GetName(), trafo.Int2ext(i, st.Vec()(i)), trafo.Int2extError(i, st.Vec()(i), err));
-         if ((*ipar).HasLowerLimit() && (*ipar).HasUpperLimit())
-            SetLimits((*ipar).GetName(), (*ipar).LowerLimit(), (*ipar).UpperLimit());
-         else if ((*ipar).HasLowerLimit() && !(*ipar).HasUpperLimit())
-            SetLowerLimit((*ipar).GetName(), (*ipar).LowerLimit());
+         Add(ipar.GetName(), trafo.Int2ext(i, st.Vec()(i)), trafo.Int2extError(i, st.Vec()(i), err));
+         if (ipar.HasLowerLimit() && ipar.HasUpperLimit())
+            SetLimits(ipar.GetName(), ipar.LowerLimit(), ipar.UpperLimit());
+         else if (ipar.HasLowerLimit() && !ipar.HasUpperLimit())
+            SetLowerLimit(ipar.GetName(), ipar.LowerLimit());
          else
-            SetUpperLimit((*ipar).GetName(), (*ipar).UpperLimit());
+            SetUpperLimit(ipar.GetName(), ipar.UpperLimit());
       } else {
-         unsigned int i = trafo.IntOfExt((*ipar).Number());
+         unsigned int i = trafo.IntOfExt(ipar.Number());
          double err =
             st.Error().IsValid() ? std::sqrt(2. * up * st.Error().InvHessian()(i, i)) : st.Parameters().Dirin()(i);
-         Add((*ipar).GetName(), st.Vec()(i), err);
+         Add(ipar.GetName(), st.Vec()(i), err);
       }
    }
 
@@ -167,12 +146,8 @@ MnUserParameterState::MnUserParameterState(const MinimumState &st, double up, co
    if (fCovarianceValid) {
       fCovariance = trafo.Int2extCovariance(st.Vec(), st.Error().InvHessian());
       fIntCovariance =
-         MnUserCovariance(std::vector<double>(st.Error().InvHessian().Data(),
-                                              st.Error().InvHessian().Data() + st.Error().InvHessian().size()),
-                          st.Error().InvHessian().Nrow());
+         MnUserCovariance({st.Error().InvHessian().Data(), st.Error().InvHessian().size()}, st.Error().InvHessian().Nrow());
       fCovariance.Scale(2. * up);
-      fGlobalCC = MnGlobalCorrelationCoeff(st.Error().InvHessian());
-      fGCCValid = fGlobalCC.IsValid();
 
       assert(fCovariance.Nrow() == VariableParameters());
 
@@ -243,7 +218,6 @@ void MnUserParameterState::Add(const std::string &name, double val, double err)
    if (fParameters.Add(name, val, err)) {
       fIntParameters.push_back(val);
       fCovarianceValid = false;
-      fGCCValid = false;
       fValid = true;
    } else {
       // redefine an existing parameter
@@ -268,7 +242,6 @@ void MnUserParameterState::Add(const std::string &name, double val, double err,
    if (fParameters.Add(name, val, err, low, up)) {
       fCovarianceValid = false;
       fIntParameters.push_back(Ext2int(Index(name), val));
-      fGCCValid = false;
       fValid = true;
    } else { // Parameter already exist - just set values
       int i = Index(name);
@@ -294,6 +267,37 @@ void MnUserParameterState::Add(const std::string &name, double val)
       SetValue(name, val);
 }
 
+void MnUserParameterState::AddCovariance(const MnUserCovariance &cov)
+{
+
+   unsigned int nrow = VariableParameters();
+   assert(cov.Nrow() >= nrow);
+
+   // add external covariance matrix
+   fCovariance = cov;
+
+   // compute and add internal covariance matrix
+   MnUserCovariance covsqueezed;
+   if (cov.Nrow() > nrow)
+      covsqueezed = MnCovarianceSqueeze()(cov, nrow);
+   else if (cov.Nrow() == nrow)
+      covsqueezed = cov;
+
+   MnAlgebraicVector params(nrow);
+   for (unsigned int i = 0; i < nrow; i++)
+      params(i) = fParameters.Params()[i];
+
+   MnAlgebraicSymMatrix covmat(nrow);
+   for (unsigned int i = 0; i < nrow; i++)
+      for (unsigned int j = i; j < nrow; j++)
+         covmat(i, j) = covsqueezed(i, j);
+
+   fIntCovariance = fParameters.Trafo().Ext2intCovariance(params, covmat);
+   fIntCovariance.Scale(0.5); //?
+   fCovarianceValid = true;
+   fCovStatus = 0; //?
+}
+
 // interaction via external number of Parameter
 
 void MnUserParameterState::Fix(unsigned int e)
@@ -310,7 +314,6 @@ void MnUserParameterState::Fix(unsigned int e)
       fIntParameters.erase(fIntParameters.begin() + i, fIntParameters.begin() + i + 1);
    }
    fParameters.Fix(e);
-   fGCCValid = false;
 }
 
 void MnUserParameterState::Release(unsigned int e)
@@ -321,7 +324,6 @@ void MnUserParameterState::Release(unsigned int e)
       return;
    fParameters.Release(e);
    fCovarianceValid = false;
-   fGCCValid = false;
    unsigned int i = IntOfExt(e);
    if (Parameter(e).HasLimits())
       fIntParameters.insert(fIntParameters.begin() + i, Ext2int(e, Parameter(e).Value()));
@@ -353,7 +355,6 @@ void MnUserParameterState::SetLimits(unsigned int e, double low, double up)
    // set limits for parameter e (external index)
    fParameters.SetLimits(e, low, up);
    fCovarianceValid = false;
-   fGCCValid = false;
    if (!Parameter(e).IsFixed() && !Parameter(e).IsConst()) {
       unsigned int i = IntOfExt(e);
       if (low < fIntParameters[i] && fIntParameters[i] < up)
@@ -370,7 +371,6 @@ void MnUserParameterState::SetUpperLimit(unsigned int e, double up)
    // set upper limit for parameter e (external index)
    fParameters.SetUpperLimit(e, up);
    fCovarianceValid = false;
-   fGCCValid = false;
    if (!Parameter(e).IsFixed() && !Parameter(e).IsConst()) {
       unsigned int i = IntOfExt(e);
       if (fIntParameters[i] < up)
@@ -385,7 +385,6 @@ void MnUserParameterState::SetLowerLimit(unsigned int e, double low)
    // set lower limit for parameter e (external index)
    fParameters.SetLowerLimit(e, low);
    fCovarianceValid = false;
-   fGCCValid = false;
    if (!Parameter(e).IsFixed() && !Parameter(e).IsConst()) {
       unsigned int i = IntOfExt(e);
       if (low < fIntParameters[i])
@@ -400,7 +399,6 @@ void MnUserParameterState::RemoveLimits(unsigned int e)
    // remove limit for parameter e (external index)
    fParameters.RemoveLimits(e);
    fCovarianceValid = false;
-   fGCCValid = false;
    if (!Parameter(e).IsFixed() && !Parameter(e).IsConst())
       fIntParameters[IntOfExt(e)] = Value(e);
 }
@@ -524,6 +522,21 @@ void MnUserParameterState::SetPrecision(double eps)
    fParameters.SetPrecision(eps);
 }
 
+MnGlobalCorrelationCoeff MnUserParameterState::GlobalCC() const
+{
+   if (!fCovarianceValid) {
+      return MnGlobalCorrelationCoeff{}; // invalidate
+   }
+   int n = fIntCovariance.Nrow();
+   MnAlgebraicSymMatrix invHessian{static_cast<unsigned int>(n)};
+   for (int i = 0; i < n; ++i) {
+      for (int j = 0; j <= i; ++j) {
+         invHessian(i, j) = fIntCovariance(i, j);
+      }
+   }
+   return MnGlobalCorrelationCoeff{invHessian};
+}
+
 } // namespace Minuit2
 
 } // namespace ROOT
