@@ -13,10 +13,11 @@
 #include "Minuit2/GradientCalculator.h"
 #include "Minuit2/MnMachinePrecision.h"
 #include "Minuit2/MnLineSearch.h"
-#include "Minuit2/MnParabolaPoint.h"
 #include "Minuit2/VariableMetricEDMEstimator.h"
 #include "Minuit2/MnPrint.h"
 
+#include "Math/Util.h"
+
 #include <cmath>
 
 namespace ROOT {
@@ -34,6 +35,9 @@ MinimumState NegativeG2LineSearch::operator()(const MnFcn &fcn, const MinimumSta
    //
    MnPrint print("NegativeG2LineSearch");
 
+   // Print the runtime on returning from the function
+   ROOT::Math::Util::TimingScope timingScope([&print](std::string const& s){ print.Info(s); }, "Done after");
+
    bool negG2 = HasNegativeG2(st.Gradient(), prec);
    if (!negG2)
       return st;
@@ -85,7 +89,7 @@ MinimumState NegativeG2LineSearch::operator()(const MnFcn &fcn, const MinimumSta
             print.Debug("Iter", iter, "param", i, pa.Vec()(i), "grad2", dgrad.G2()(i), "grad",
                         dgrad.Vec()(i), "grad step", step(i), " gdel ", gdel);
 
-            MnParabolaPoint pp = lsearch(fcn, pa, step, gdel, prec);
+            auto pp = lsearch(fcn, pa, step, gdel, prec);
 
             print.Debug("Line search result", pp.X(), "f(0)", pa.Fval(), "f(1)", pp.Y());
 
