@@ -28,13 +28,10 @@ operator()(const MnFcn &fcn, const GradientCalculator &, const MnUserParameterSt
    const MnMachinePrecision &prec = st.Precision();
 
    // initial starting values
-   MnAlgebraicVector x(n);
-   for (unsigned int i = 0; i < n; i++)
-      x(i) = st.IntParameters()[i];
-   double fcnmin = fcn(x);
+   MnAlgebraicVector x(st.IntParameters());
+   double fcnmin = MnFcnCaller{fcn}(x);
    MinimumParameters pa(x, fcnmin);
-   InitialGradientCalculator igc(fcn, st.Trafo());
-   FunctionGradient dgrad = igc(pa);
+   FunctionGradient dgrad = calculateInitialGradient(pa, st.Trafo(), fcn.ErrorDef());
    MnAlgebraicSymMatrix mat(n);
    double dcovar = 1.;
    for (unsigned int i = 0; i < n; i++)
@@ -46,13 +43,6 @@ operator()(const MnFcn &fcn, const GradientCalculator &, const MnUserParameterSt
    return MinimumSeed(state, st.Trafo());
 }
 
-MinimumSeed SimplexSeedGenerator::operator()(const MnFcn &fcn, const AnalyticalGradientCalculator &gc,
-                                             const MnUserParameterState &st, const MnStrategy &stra) const
-{
-   // base class interface
-   return (*this)(fcn, (const GradientCalculator &)(gc), st, stra);
-}
-
 } // namespace Minuit2
 
 } // namespace ROOT
