@@ -9,9 +9,8 @@ void bind_application(py::module m) {
   py::class_<MnApplication>(m, "MnApplication")
 
       .def("__call__", &MnApplication::operator())
-      .def_property("precision", &MnApplication::Precision,
-                    &MnApplication::SetPrecision)
       .def_property_readonly("strategy", &MnApplication::Strategy)
-
-      ;
+      .def_property(
+          "precision", [](const MnApplication& self) { return self.Precision().Eps(); },
+          [](MnApplication& self, double eps) { self.State().SetPrecision(eps); });
 }
