@@ -1,7 +1,7 @@
+#include "pybind11.hpp"
 #include <Minuit2/FCNBase.h>
 #include <Minuit2/MnScan.h>
 #include <Minuit2/MnUserParameterState.h>
-#include <pybind11/pybind11.h>
 
 namespace py = pybind11;
 using namespace ROOT::Minuit2;
