@@ -7,7 +7,6 @@ from iminuit._core import (
     MnScan,
     FunctionMinimum,
     MnSimplex,
-    MnPrint,
     MnUserCovariance,
     MinimumState,
 )
@@ -17,16 +16,6 @@ import pickle
 import numpy as np
 
 
-@pytest.fixture
-def debug():
-    prev = MnPrint.global_level
-    MnPrint.global_level = 3
-    MnPrint.show_prefix_stack(True)
-    yield
-    MnPrint.global_level = prev
-    MnPrint.show_prefix_stack(False)
-
-
 def test_MnStrategy():
     assert MnStrategy() == 1
     assert MnStrategy(0) == 0
@@ -88,7 +77,7 @@ def test_MnUserParameterState():
 
 
 def test_MnMigrad():
-    fcn = FCN(fn, None, False, 1)
+    fcn = FCN(fn, None, None, None, False, 1)
     state = MnUserParameterState()
     state.add("x", 5, 0.1)
     state.add("y", 3, 0.2, -5, 5)
@@ -105,7 +94,7 @@ def test_MnMigrad():
 
 
 def test_MnMigrad_grad():
-    fcn = FCN(lambda x: 10 + x**2, lambda x: [2 * x], False, 1)
+    fcn = FCN(lambda x: 10 + x**2, lambda x: [2 * x], None, None, False, 1)
     state = MnUserParameterState()
     state.add("x", 5, 0.1)
     migrad = MnMigrad(fcn, state, 1)
@@ -120,6 +109,90 @@ def test_MnMigrad_grad():
     assert fcn._ngrad > 0
 
 
+def test_MnMigrad_g2():
+    # g2 is diagonal of hessian(chi2)
+    fcn = FCN(
+        lambda x: 10 + (x / 4) ** 2,
+        lambda x: [x / 8],
+        lambda x: [1 / 8],
+        None,
+        False,
+        1,
+    )
+    state = MnUserParameterState()
+    state.add("x", 5, 0.1)
+    migrad = MnMigrad(fcn, state, 1)
+    fmin = migrad(0, 0.1)
+    state = fmin.state
+    assert len(state) == 1
+    assert state[0].number == 0
+    assert state[0].name == "x"
+    assert state[0].value == approx(0, abs=1e-3)
+    assert state[0].error == approx(4, abs=1e-3)
+    assert fcn._nfcn > 0
+    assert fcn._ngrad > 0
+    assert fcn._ng2 > 0
+
+
+def test_MnMigrad_hessian():
+    fcn = FCN(
+        lambda x: 10 + (x / 4) ** 2,
+        lambda x: [x / 8],
+        None,
+        lambda x: [[1 / 8]],
+        False,
+        1,
+    )
+    state = MnUserParameterState()
+    state.add("x", 5, 0.1)
+    migrad = MnMigrad(fcn, state, 1)
+    fmin = migrad(0, 0.1)
+    state = fmin.state
+    assert len(state) == 1
+    assert state[0].number == 0
+    assert state[0].name == "x"
+    assert state[0].value == approx(0, abs=1e-3)
+    assert state[0].error == approx(4, abs=1e-3)
+    assert fcn._nfcn > 0
+    assert fcn._ngrad > 0
+    assert fcn._ng2 == 0
+    assert fcn._nhessian > 0
+
+
+def test_MnMigrad_g2_hessian():
+    # if both g2 and hessian are available, hessian is used
+    fcn = FCN(
+        lambda x, y: 10 + (2 * x) ** 2 + ((y - 1) / 4) ** 2,
+        lambda x, y: [8 * x, (y - 1) / 8],
+        lambda x, y: [-1, -1],  # not used
+        lambda x, y: [[8.0, 0], [0, 1 / 8]],
+        False,
+        1,
+    )
+    # cov = inv(hessian / 2) = [[1/4, 0], [0, 16]]
+    # err[0] = cov[0, 0]**0.5 = 0.5
+    # err[1] = cov[1, 1]**0.5 = 4
+    state = MnUserParameterState()
+    state.add("x", 5, 0.1)
+    state.add("y", 5, 0.1)
+    migrad = MnMigrad(fcn, state, 1)
+    fmin = migrad(0, 0.1)
+    state = fmin.state
+    assert len(state) == 2
+    assert state[0].number == 0
+    assert state[0].name == "x"
+    assert state[0].value == approx(0, abs=1e-2)
+    assert state[0].error == approx(0.5, abs=1e-2)
+    assert state[1].number == 1
+    assert state[1].name == "y"
+    assert state[1].value == approx(1, abs=1e-2)
+    assert state[1].error == approx(4, abs=1e-2)
+    assert fcn._nfcn > 0
+    assert fcn._ngrad > 0
+    assert fcn._ng2 == 0
+    assert fcn._nhessian > 0
+
+
 def test_MnMigrad_cfunc():
     nb = pytest.importorskip("numba")
 
@@ -135,7 +208,8 @@ def test_MnMigrad_cfunc():
             r += (y[i] - x[i]) ** 2
         return r
 
-    fcn = FCN(fcn, None, True, 1)
+    fcn = FCN(fcn, None, None, None, True, 1)
+    assert fcn._cfcn is True
     state = MnUserParameterState()
     for i in range(len(y)):
         state.add(f"x{i}", 5, 0.1)
@@ -153,6 +227,8 @@ def test_MnMigrad_np():
     fcn = FCN(
         lambda xy: 10 + xy[0] ** 2 + ((xy[1] - 1) / 2) ** 2,
         lambda xy: [2 * xy[0], (xy[1] - 1)],
+        None,
+        None,
         True,
         1,
     )
@@ -178,7 +254,7 @@ def test_MnMigrad_np():
 
 
 def test_MnScan():
-    fcn = FCN(lambda x: 10 + x**2, None, False, 1)
+    fcn = FCN(lambda x: 10 + x**2, None, None, None, False, 1)
     state = MnUserParameterState()
     state.add("x", 2, 5)
     scan = MnScan(fcn, state, 1)
@@ -190,7 +266,7 @@ def test_MnScan():
 
 
 def test_MnSimplex():
-    fcn = FCN(lambda x: 10 + x**2, None, False, 1)
+    fcn = FCN(lambda x: 10 + x**2, None, None, None, False, 1)
     state = MnUserParameterState()
     state.add("x", 2, 5)
     simplex = MnSimplex(fcn, state, 1)
@@ -202,7 +278,7 @@ def test_MnSimplex():
 
 
 def test_FunctionMinimum():
-    fcn = FCN(lambda x: 10 + x**2, None, False, 1)
+    fcn = FCN(lambda x: 10 + x**2, None, None, None, False, 1)
     st = MnUserParameterState()
     st.add("x", 0.01, 5)
     str = MnStrategy(1)
@@ -218,7 +294,7 @@ def test_FunctionMinimum_pickle():
     st = MnUserParameterState()
     st.add("x", 1, 0.1)
     st.add("y", 2, 0.1, 1, 3)
-    fm = FunctionMinimum(FCN(fn, None, False, 1), st, 1, 0.1)
+    fm = FunctionMinimum(FCN(fn, None, None, None, False, 1), st, 1, 0.1)
 
     pkl = pickle.dumps(fm)
     fm2 = pickle.loads(pkl)
